# -*- coding: utf-8 -*-
import cocos
from cocos.actions import *
import data
import pyglet

from cocos.rect import Rect
from mapsprite import MapSprite

class BuildingSprite(MapSprite):
    def __init__(self,*args,**kwds):
        MapSprite.__init__(self,*args,**kwds)
        self.gunDamage = 10
        self.gunRange = 2
        self.gunSpeed = 0.75
        self.reloadProgress = 1. / self.gunSpeed
        self.scaleBox(20,20)
        self.createRect()
        self.calcRangeRect()
    
    def __repr__(self):
        return "<Building @(%d,%d)>" % self.position
        
    def withinRange(build,bug):
        return build.range_rect.intersects(bug.rect)
        
    def contains(self,x,y):
        """
        Overriding the default contains method for performance reasons.
        """
        return self.rect.contains(x,y)
    def calcRangeRect(self):
        #create range sqaure arround buildings
        scale = self.gunRange
        x,y   = self.position
        bl    = self.rect.get_bottomleft()
        br    = self.rect.get_bottomright()
        tl    = self.rect.get_topleft()
        hoog  = tl[1] - bl[1]
        breed = br[0] - bl[0]
        
        size_new = breed * scale
        #hoog_new  = hoog  * scale
        
        bl_new = (x - size_new /2,y - size_new/2)
                    
        #bl_new = (bl[0] - scale*bl[0],bl[1] - scale*bl[1]);
        #hoog_new = hoog + scale*hoog
        #breed_new = breed + scale*breed
            
        self.range_rect = Rect(bl_new[0],bl_new[1],size_new,size_new)
        
    def shoot(self,bug):
        gunDamage = 10
        if self.canShoot():
            self.reloadProgress = 0.0
            return bug.doDamage(gunDamage)
        else:
            return False
            
            
    def canShoot(self):
        """
        Returns True if the gun is reloaded and ready to shoot.
        """
        if self.reloadProgress >= (1. / self.gunSpeed):
            return True
        else:
            return False
        
    def madeReloadProgress(self,dt):
        self.reloadProgress += dt
        
class SprayBuilding(BuildingSprite):
    building_pic = 'bugspray.png'
    def __init__(self,*args,**kwds):
        BuildingSprite.__init__(self,*args,**kwds)
        self.gunDamage = 8
        self.gunRange = 2
        self.gunSpeed = 4
        self.reloadProgress = 1. / self.gunSpeed
        self.bullet_pic = 'bugspray_bullet.png'
        
class UvBuilding(BuildingSprite):
    building_pic = 'zapper.png'
    def __init__(self,*args,**kwds):
        BuildingSprite.__init__(self,*args,**kwds)
        self.gunDamage = 6
        self.gunRange = 4
        self.gunSpeed = 3
        self.reloadProgress = 1. / self.gunSpeed
        self.bullet_pic = 'zapper_bullet.png'
        
class HoseBuilding(BuildingSprite):
    building_pic = 'hose.png'
    def __init__(self,*args,**kwds):
        BuildingSprite.__init__(self,*args,**kwds)
        self.gunDamage = 4
        self.gunRange = 6
        self.gunSpeed = 2
        self.reloadProgress = 1. / self.gunSpeed
        self.bullet_pic = 'hose_bullet.png'