# -*- coding: utf-8 -*-
"""
Contains the main map/engine class. This class *should* be the only interface
between layers and the map.
"""
import math
import AStar

from math import *
import data

# Some global declarations.
SCREEN_X = 640
SCREEN_Y = 480
DIFF_X   = 32
DIFF_Y   = 24

class GameMap():
    def __init__(self):
        """
        self.sprites is a python dictionary that will contain references to all
        the objects on the map.
        """
        self.M = 24
        self.N = 32

        self.H = 480
        self.W = 640
        
        self.sprites = {}
        self.sprites['bugs'] = []
        self.imageCache = {}
        self.mesh = self.generateMesh(640,480,32,24)
        
        
        # Defining values related to the scoring system.
        self.xp     = 0
        self.hp = 100
        self.lvl  = 1
        self.bugScale = 1
        self.ss = 1000
    def status(self):
        print 'HP = %d, XP = %d, LVL = %d, $$ = %d' % (self.hp,
                        self.xp,self.lvl,self.ss)
        
    def increaseSS(self,cash):
        self.ss += cash
        
    def decreaseSS(self,cash):
        self.ss -= cash
        
    def increaseXP(self,score):
        self.xp += score
        self.status()
        
    def updateBugs(self):
        """
        Updates the bugs. Checks to see if the bugs are on the main path, if
        not it calculates a new individual path for the specific bug to get
        back onto the main path.
        """
        
        path    = self.path
        
        for b in self.getBugs():
            x,y = b.position
            BugElement = self.coordToIndex(x,y)
            
            if BugElement in path:
                b.followMainPath = True
                b.mainPathIndex = path.index(BugElement)
                
            else:
                b.followMainPath = False
                
                # First find the closest element on the main path to the bugs
                # current position.
                
                coords = [self.indexToCenter(p) for p in path]
                distances = [sqrt((x - X[0])**2+ (y - X[1])**2) for X in coords]
                
                destElementIndex = distances.index(min(distances))
                destElement = path[destElementIndex]
                
                startElement = self.coordToIndex(x,y)
                
                # We now have the starting and destination element of the
                # current bug, we now need to calculate the path between the
                # start and finish.
                b.ownPath = self.calcBugPath(b,startElement,destElement)
                #print 'My own path =',
                #print b.ownPath
                b.ownPathIndex = 0
    
    def calcBugPath(self,bug,start,dest):
        """
        Calculates a bugs path to the main path.
        """
        startCoord = self.indexToCenter(start)
        destCoord  = self.indexToCenter(dest)
        path = self.findPathlines(32,24,640,480,startCoord,destCoord)
        return path

    def meshToIndex(self,m,n):
        """
        Converts grid points to an element number.
        """
        return self.N * m + n
        
    def coordToCentre(self,x,y):
        """
        Given coordinates on screen this function returns the centre of 
        element belonging to the coordinates.
        """
        i = self.coordToIndex(x,y)
        return self.indexToCenter(i)

    def coordToIndex(self,x,y):
        """
        Returns the element number contains the point (x,y)
        
        Adolph: Checked.
        """
        node = self.NodeLocate(x,y,self.N,self.M,self.W,self.H)
        n = node[0]
        m = node[1]
        i = self.N * m + n
        #print "Index= ",str(i)
        return i
    
    def indexToCenter(self,i):
        """
        Takes an element number and calculates the center of that element.
        """
        i = int(i)
        n = i % self.N
        m = i / self.N
        x = n * 20 + 10
        y = m * 20 + 10
        #print "grid coord = ",str((m,n))
        #print "x,y = ",str((x,480 - y))
        return (x,480 - y)
        
    def loadImage(self,filename):
        if self.imageCache.has_key(filename):
            return self.imageCache[filename]
        else:
            image = data.loadImage(filename)
            return self.imageCache.setdefault(filename,image)
        
    def getElementCenter(self,x,y):
        """Returns the center of the element that contains this point."""
        #print (x,y)
        node = self.NodeLocate(x,y,32,24,640,480)
        #print node
        m = int(node[0])
        n = int(node[1])
        i = 32*n + m + 1
        x = m * 20 + 10
        y = n * 20 + 10
        #print (x,y)
        return (x,480 - y)
    def addBug(self,bug):
        """
        Adds a bug to the list of bugs
        """
        #self.sprites.setdefault('bugs',[]).append(bug)
        self.sprites['bugs'].append(bug)
        
    def removeBuilding(self,building):
        self.sprites['buildings'].remove(building)
        
    def addBuilding(self,building):
        self.sprites.setdefault('buildings',[]).append(building)
        
    def removeBug(self,bug):
        """
        Remove bug from the list of bugs.
        """
        try:
            self.sprites['bugs'].remove(bug)
        except:
            pass

    def getBugs(self):
        """
        Returns a list of bugs.
        """
        return self.sprites.get('bugs',[])
        
        
    def getBuildings(self):
        return self.sprites.get('buildings',[])
    
    
    def generateMesh(self,screen_sizex,screen_sizey,m,n):
        '''
        generates the nodal co-ordinate matrix of the mesh 
        '''
        dx = screen_sizex/m;
        dy = screen_sizey/n;
        matrx = [];
        matry = [];
        x_pos = dx/2;
        y_pos = dy/2;
        for j in xrange(1,n+1):
            for i in xrange(1,m+1):
                matrx.append(x_pos);
                matry.append(y_pos);
                x_pos = x_pos + dx;
            y_pos = y_pos + dy;
            x_pos = dx/2;
        return (matrx,matry)
            
    def NodeLocate(self,x_coord,y_coord,m,n,width,heigth):
        '''
        given an x and y location, this function determines 
        the mesh element in which the co-ordinate is located
        '''
        node = [];
        node.append(floor(float(x_coord)/width*m))
        node.append((n-1)-floor(float(y_coord)/heigth*n))
        return node
    
    
    
    def findPathlines(self,m,n,width,height,startpos,endpos):
        '''
        generates a vector that defines where in the map buildings are located.
        NOTE: pos (0,0) is @top left corner
         -1:building,  1: walkable     5: starting point    6:end point        
         
        '''
        bugs = self.getBugs()
        #if len(bugs)>0:
        if True:
            #startpos = bugs[0].position
            
            temp=[1]
            mapData = temp*m*n
            build = self.getBuildings()
            #matrx,matry = self.generateMesh(width,height,m,n)
            
            # Adolph: Here we can maybe have each building save its own element
            # number, which should speed up the generation of mapData. Also we
            # might want to save mapData and each time a building is placed
            # only update that element of mapData.
            
            matrx,matry = self.mesh
            for i in xrange(len(matrx)):
                x = matrx[i]
                y = matry[i]
                for b in build:
                    c = b.contains(x,y)
                    if c:
                        node = self.NodeLocate(x,y,m,n,width,height)
                        a = int(node[0] + node[1]*(m))
                        mapData[a] = -1
                        
            # Define the starting point.
            node_start = self.NodeLocate(startpos[0],startpos[1],m,n,width,height)
            a = int(node_start[0] + node_start[1]*(m))
            mapData[a] = 5
            
            #Define the end point.
            node_end = self.NodeLocate(endpos[0],endpos[1],m,n,width,height)
            a = int(node_end[0] + node_end[1]*(m))
            mapData[a] = 6
            
            
            # Adolph: I changed the path calculation to only save the element
            # number instead of the actual coordinates, I suspect this will
            # perform better.
            
            #===============================
            #going to try and find the path here for now
            astar = AStar.AStar(AStar.SQ_MapHandler(mapData,m,n))
            start = AStar.SQ_Location(node_start[0],node_start[1])
            end = AStar.SQ_Location(node_end[0],node_end[1])
            
            p = astar.findPath(start,end)
            #print 'Found path:'
            #for n in p.nodes:
            #    print n.lid
            
            #dx = width/m;
            #dy = height/n;
            
            if not p:
                print "No path found!"
                pathLine = []
            else:
                #print "Found path in %d moves." % (len(p.nodes))
                pathLine = [i.lid for i in p.nodes]
                #pathlines = []
                #pathlines.append((start.x*dx+dx/2,(height-start.y*dy)+dy/2))
                #for n in p.nodes:
                #    pathlines.append((n.location.x*dx+dx/2,(height-n.location.y*dy)+dy/2))
                #pathlines.append((end.x*dx+dx/2,(height-end.y*dy)+dy/2))
            
            #===================================

            #self.path = pathLine
            print "The path = ",
            print pathLine
            return pathLine
            
            
    def findMainpath(self):
        bugs = self.getBugs()
        #if len(bugs)>0:
        if True:
            self.mainpath = self.findPathlines(32,24,640,480,[10,220],[600,220])
            self.path = self.mainpath # Just for backwards compatibility.
            if self.mainpath == []:
                return False
            else:
                return True
