# -*- coding: utf-8 -*-



import cocos
from cocos.actions import *
import data
import pyglet


from mapsprite import MapSprite
from bugsprite import BugSprite
from buildingsprite import BuildingSprite

class GunLayer(cocos.layer.Layer):
    """
    GunLayer shoots at bugs
    """
    def __init__(self,gameMap):
        """
        This function is called when the layer is CREATED.
        """
        super(GunLayer, self ).__init__()
        self.gameMap = gameMap
                  

        self.bullets = []
        # Schedule an interval at which we check to see if a tower can
        # shoot a bug.
        self.schedule_interval(self.shootBugs,1./1.4)
    
    def shootBugs(self,dt):
        """
        This function calculates what bugs to shoot
        """
        buildings = self.gameMap.getBuildings()
        bugs      = self.gameMap.getBugs()

        deadBugs = []
        for build in buildings:
            
            # For each building we update the time since last fired.
            build.madeReloadProgress(dt)
            
            # If the tower has reloaded, canShoot() will be true, we only loop
            # through the bugs if the current tower is ready to fire.
            if build.canShoot():
                #print build
                for bug in bugs:
                    
                    #print bug
                    bug.createRect()
                    if build.withinRange(bug):

                        #draw bullet moving to target
                        buildc = build.position
                        bugc   = bug.position
                        
                        bullet = MapSprite(build.bullet_pic,
                                            position=(buildc[0],buildc[1]))
                        self.add(bullet,z=2)
                        #self.bullets.append(bullet)
                        move = MoveTo((bugc[0],bugc[1]),
                                        duration = 0.5)
                        explode = ScaleTo(0.04,0.2) | FadeOut(0.6)
                        delSprite = CallFuncS(self.removeSprite)
                        bullet.do(move + delSprite)

                        # Next we fire upon a bug, if the bug is killed we add
                        # it to the list of dead bugs, so that we can remove them
                        # after the loop.
                        if build.shoot(bug):
                            self.gameMap.increaseXP(bug.xp)
                            self.gameMap.increaseSS(5)
                            if bug not in deadBugs:
                                deadBugs.append(bug)
                        break
                        
        # After all the buildings have fired we remove all the collected dead
        # bugs
        for bug in deadBugs:
            self.gameMap.removeBug(bug)
            bug.parent.remove(bug)
            
    def removeSprite(self,doomedSprite):
        doomedSprite.parent.remove(doomedSprite)
        del doomedSprite
        #doomedSprite.delete()