# -*- coding: utf-8 -*-

"""
This is the template layer it should be used as a base for all the other layers
of the game. Copy it and rename it.
"""

# Importing all the required modules.
import cocos
from cocos.actions import *
import data
import pyglet
from cocos import director

from mapsprite import MapSprite
from buildingsprite import *

class HudLayer(cocos.layer.Layer):
    # This is needed to enable event handling for this layer.
    is_event_handler = True

    def __init__(self,gameMap):
        super(HudLayer, self ).__init__()
        
        self.shown = True
        # Add the grass hud.
        grasshud = MapSprite('grassvector.png',
                            opacity=75,
                            position=(350,150))
        
        self.add(grasshud)
        self.grasshud = grasshud
        
        hudshow = MapSprite('hud_show.png',
                    opacity=155,
                    position=(25,25))
        
        self.add(hudshow)
        self.hudshow = hudshow
        
        

        self.gameMap = gameMap


        listOfBuildings = [SprayBuilding,UvBuilding,HoseBuilding]
        self.listOfBuildings = listOfBuildings
                            
        self.hudIcons = []

        xOffset = 75

        for b in listOfBuildings:
            buildIcon = MapSprite(b.building_pic,position=(xOffset,25))

            buildIcon.makeSize(30,30)
            xOffset += 45
            self.add(buildIcon)
            self.hudIcons.append(buildIcon)
            buildIcon.getRect()
            
            
        # Creating a self.select flag.
        self.select = False
        
        
        
        # Displaying the score etc on the screen.
        self.hp = cocos.text.Label('HP = 100',
                                    position=(500,450),
                                    color=(255,255,255,255))
        self.xp = cocos.text.Label('XP = 0 / 100',
                                    position=(500,430),
                                    color=(255,255,255,255))
                                    
        self.ss = cocos.text.Label('$ = 100',
                                    position=(500,410),
                                    color=(255,255,255,255))
               
        self.add(self.hp)
        self.add(self.xp)
        self.add(self.ss)
        
        self.schedule_interval(self.updateDisplay,1)

    def trueXY(self,x,y):
        return cocos.director.director.get_virtual_coordinates(x,y)
        
    def hideHud(self):
        hideMe = MoveBy((0,-150),1)
        self.grasshud.do(hideMe)
        for i in self.hudIcons: i.do(hideMe)
        self.shown = False
        
    def showHud(self):
        showMe = MoveBy((0,150),1)
        self.grasshud.do(showMe)
        for i in self.hudIcons: i.do(showMe)
        self.shown = True
        
    def updateDisplay(self,dt):
        self.xp.element.text = 'XP = ' + str(self.gameMap.xp)
        self.hp.element.text = 'HP = ' + str(self.gameMap.hp)
        self.ss.element.text = '$$ = ' + str(self.gameMap.ss)
    def on_mouse_press(self,x,y,buttons,modifiers):
        
        X,Y = cocos.director.director.get_virtual_coordinates(x,y)
        
        i = self.gameMap.coordToIndex(X,Y)
        print 'Element center of point clicked = ',
        print self.gameMap.indexToCenter(i)
        if self.hudshow.contains(X,Y):
            if self.shown:
                self.hideHud()
            else:
                self.showHud()
                
        
        
        if self.select:
            buildings = self.gameMap.getBuildings()
            x,y = self.selectedStructure.position
            selectedElement = self.gameMap.coordToIndex(x,y)
            for b in buildings:
                x,y = b.position
                bElement = self.gameMap.coordToIndex(x,y)
                if bElement == selectedElement:
                    print "Cannot place buildings on top of each other."
                    return
            
            
            #X,Y = self.gameMap.getElementCenter(X,Y)
            X,Y = self.gameMap.coordToCentre(X,Y)
            self.selectedStructure.do( MoveTo((X,Y),0))
            
            self.gameMap.addBuilding(self.selectedStructure)
            self.selectedStructure.createRect()
            
            

            
            
            if not self.gameMap.findMainpath():
                self.select = True
                print "Cannot block path."
                self.gameMap.removeBuilding(self.selectedStructure)
                return
            elif self.gameMap.ss <= 10:
                print "Not enough money"
                self.select = True
                self.gameMap.removeBuilding(self.selectedStructure)
            else:
                #self.selectedStructure.createRect()
                self.selectedStructure.calcRangeRect()
                self.selectedStructure.opacity = 255
                self.gameMap.updateBugs()
                self.select = False
                self.gameMap.decreaseSS(30)
            return
            
            
        for i,icon in enumerate(self.hudIcons):
            icon.createRect()
            if icon.contains(X,Y):
                self.select = True
                buildClass = self.listOfBuildings[i]
                print buildClass.building_pic
                self.selectedStructure = buildClass(buildClass.building_pic)
                #self.selectedStructure =
                #BuildingSprite(self.listOfBuildings[i])
                #self.selectedStructure.makeSize(50,50)
                self.selectedStructure.opacity = 100
                #self.gameMap.addBuilding(self.selectedStructure)
                self.add(self.selectedStructure)
                
                print "Clicked on ", str(icon)
                
                
    def on_mouse_motion(self,x,y,dx,dy):
        if self.select:
            x,y = self.trueXY(x,y)
            self.selectedStructure.do( MoveTo((x,y), 0 ))
                
        
        
    