# -*- coding: utf-8 -*-
import cocos
from cocos.actions import *
from cocos.rect import Rect
import data
import pyglet



class MapSprite(cocos.sprite.Sprite):
    def __init__(self,spriteImage,*args,**kwds):
        
        imageData = data.loadImage(spriteImage)
        
        cocos.sprite.Sprite.__init__(self,imageData,*args,**kwds)
        self.createRect()
    
    def scaleBox(self,width,height):
        """
        Fixes the error made in makeSize.
        """
        widthScaling = float(width) / self.width
        heightScaling = float(height) / self.height
        
        if widthScaling < heightScaling:
            scaling = widthScaling
        else:
            scaling = heightScaling
        self.do(ScaleBy(scaling,0))
        
    def createRect(self):
        # We now create a rectangle instance for the current sprite
        # the rectangle allows us to more easily use the space taken
        # up by the current sprite.
        x,y = self.position
        xbl = x - self.width / 2
        ybl = y - self.height / 2
        
        self.rect = Rect(xbl,ybl,self.width,self.height)
        
    def getRect(self):
        return self.rect
        
    def contains(self,x,y):
        """
        Checks to see wheter the x,y point is inside this sprite.
        """
        self.createRect()
        return self.rect.contains(x,y)
        
        
    def intersects(self,other):
        """
        Checks to see whether or not the other object.
        """
        self.createRect()
        other.createRect()
        return self.rect.intersects(other.rect)
        
    def getIntersects(self,lOther):
        """
        Returns a list of objects intersecting.
        """
        intersects = []
        for o in lOther:
            if self.intersects(o):
                intersects.append(o)
                
        return intersects
        
    def getWidth(self):
        """Returns the scaled with of the sprite."""
        return self.width
    
    def getHeight(self):
        """Returns the scaled width of the sprite."""
        return self.height
        
        
    def makeSize(self,width,height):
        """
        Because we are using images of different sizes, their width
        and height might not be what is required. This method scales
        the sprite so that its width and height are not larger than
        maxHeight and maxWidth.
        """
        #print self.width
        #print self.height
        if width > height:
            #print "width > height"
            scaling = float(width) / self.width
        else:
            #print "height > width"
            scaling = float(height) / self.height
        #print scaling
        self.do( ScaleBy(scaling,0))
        
        
        